@@ -26,6 +26,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <poll.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -34,10 +35,24 @@
 #include <linux/tty.h>
 #include <linux/kd.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #ifdef HAVE_PATHS_H
 #include <paths.h>
 #endif /* HAVE_PATHS_H */
 
+#ifdef HAVE_SYS_MOUNT_H
+#include <sys/mount.h>
+#endif
+
+#ifdef HAVE_SELINUX
+#include <selinux/selinux.h>
+#include <selinux/label.h>
+#include <selinux/get_default_type.h>
+#include <selinux/context.h>
+#endif
+
 #include "ck-sysdeps.h"
 
 #ifndef ERROR
@@ -93,12 +108,16 @@ typedef struct tty_map_node {
         guint major_number;
         guint minor_first;
         guint minor_last;
-        char name[16];
+        char name[32];
         char devfs_type;
 } tty_map_node;
 
 static tty_map_node *tty_map = NULL;
 
+#ifdef HAVE_SELINUX
+static struct selabel_handle *label_hnd = NULL;
+#endif
+
 /* adapted from procps */
 /* Load /proc/tty/drivers for device name mapping use. */
 static void
@@ -362,21 +381,21 @@ stat2proc (const char    *S,
         S = tmp + 2;                 /* skip ") " */
 
         num = sscanf (S,
-                      "%c "
-                      "%d %d %d %d %d "
-                      "%lu %lu %lu %lu %lu "
+                      "%c " /* state */
+                      "%d %d %d %d %d " /* ppid pgrp session tty tpgid */
+                      "%lu %lu %lu %lu %lu " /* flags min_flt cmin_flt maj_flt cmaj_flt */
                       "%Lu %Lu %Lu %Lu "  /* utime stime cutime cstime */
-                      "%ld %ld "
-                      "%d "
-                      "%ld "
+                      "%ld %ld " /* priority nice*/
+                      "%d " /* nlwp */
+                      "%ld " /* alarm */
                       "%Lu "  /* start_time */
-                      "%lu "
-                      "%ld "
-                      "%lu %"KLF"u %"KLF"u %"KLF"u %"KLF"u %"KLF"u "
+                      "%lu " /* vsize */
+                      "%ld " /* rss */
+                      "%lu %"KLF"u %"KLF"u %"KLF"u %"KLF"u %"KLF"u " /* rss_rlim start_code end_code start_stack kstk_esp kstk_eip */
                       "%*s %*s %*s %*s " /* discard, no RT signals & Linux 2.1 used hex */
-                      "%"KLF"u %*lu %*lu "
-                      "%d %d "
-                      "%lu %lu",
+                      "%"KLF"u %*u %*u " /* wchan */
+                      "%d %d " /* exit_signal processor */
+                      "%lu %lu", /* rtprio sched */
                       &P->state,
                       &P->ppid, &P->pgrp, &P->session, &P->tty, &P->tpgid,
                       &P->flags, &P->min_flt, &P->cmin_flt, &P->maj_flt, &P->cmaj_flt,
@@ -458,7 +477,7 @@ ck_unix_pid_get_env_hash (pid_t pid)
         gsize       length;
         GError     *error;
         GHashTable *hash;
-        int         i;
+        gsize       i;
         gboolean    last_was_null;
 
         g_return_val_if_fail (pid > 1, NULL);
@@ -474,7 +493,10 @@ ck_unix_pid_get_env_hash (pid_t pid)
                                    &length,
                                    &error);
         if (! res) {
-                g_warning ("Couldn't read %s: %s", path, error->message);
+                /* This is pretty harmless, usually it means the process
+                 * was short lived and we didn't get around to looking at
+                 * it before it died. */
+                g_debug ("Couldn't read %s: %s", path, error->message);
                 g_error_free (error);
                 goto out;
         }
@@ -520,7 +542,7 @@ ck_unix_pid_get_env (pid_t       pid,
         char      *val;
         gsize      length;
         GError    *error;
-        int        i;
+        guint      i;
         char      *prefix;
         int        prefix_len;
         gboolean   last_was_null;
@@ -617,6 +639,72 @@ ck_unix_pid_get_ppid (pid_t pid)
         return ppid;
 }
 
+gboolean
+ck_unix_pid_get_login_session_id (pid_t  pid,
+                                  char **idp)
+{
+        gboolean ret;
+        gboolean res;
+        char    *path;
+        char    *contents;
+        gsize    length;
+        GError  *error;
+        char    *end_of_valid_ulong;
+        gulong   ulong_value;
+
+        g_return_val_if_fail (pid > 1, FALSE);
+
+        ret = FALSE;
+        contents = NULL;
+
+        path = g_strdup_printf ("/proc/%u/sessionid", (guint)pid);
+
+        error = NULL;
+        res = g_file_get_contents (path,
+                                   &contents,
+                                   &length,
+                                   &error);
+        if (! res) {
+                g_warning ("Couldn't read %s: %s", path, error->message);
+                g_error_free (error);
+                goto out;
+        }
+
+        if (contents[0] == '\0') {
+                g_warning ("Couldn't read %s: empty file", path);
+                goto out;
+        }
+
+        errno = 0;
+        ulong_value = strtoul (contents, &end_of_valid_ulong, 10);
+
+        if (*end_of_valid_ulong != '\0') {
+                goto out;
+        }
+
+        if (errno == ERANGE) {
+                g_warning ("Couldn't read %s: %s", path, g_strerror (errno));
+                goto out;
+        }
+
+        /* Will be G_MAXUINT32 if it isn't set */
+        if (ulong_value == G_MAXUINT32) {
+                goto out;
+        }
+
+        if (idp != NULL) {
+                *idp = g_strdup_printf ("%lu", (unsigned long int)ulong_value);
+        }
+
+        ret = TRUE;
+
+ out:
+        g_free (contents);
+        g_free (path);
+
+        return ret;
+}
+
 gboolean
 ck_get_max_num_consoles (guint *num)
 {
@@ -627,6 +715,12 @@ ck_get_max_num_consoles (guint *num)
         return TRUE;
 }
 
+gboolean
+ck_supports_activatable_consoles (void)
+{
+        return TRUE;
+}
+
 char *
 ck_get_console_device_for_num (guint num)
 {
@@ -704,3 +798,403 @@ ck_get_active_console_num (int    console_fd,
 
         return ret;
 }
+
+/* adapted from upower-0.9 branch */
+static gfloat
+linux_get_used_swap (void)
+{
+        gchar *contents = NULL;
+        gchar **lines = NULL;
+        GError *error = NULL;
+        gchar **tokens;
+        gboolean ret;
+        guint active = 0;
+        guint swap_free = 0;
+        guint swap_total = 0;
+        guint len;
+        guint i;
+        gfloat percentage = 0.0f;
+        const gchar *filename = "/proc/meminfo";
+
+        /* get memory data */
+        ret = g_file_get_contents (filename, &contents, NULL, &error);
+        if (!ret) {
+                g_warning ("failed to open %s: %s", filename, error->message);
+                g_error_free (error);
+                goto out;
+        }
+
+        /* process each line */
+        lines = g_strsplit (contents, "\n", -1);
+        for (i=1; lines[i] != NULL; i++) {
+                tokens = g_strsplit_set (lines[i], ": ", -1);
+                len = g_strv_length (tokens);
+                if (len > 3) {
+                        if (g_strcmp0 (tokens[0], "SwapFree") == 0)
+                                swap_free = atoi (tokens[len-2]);
+                        if (g_strcmp0 (tokens[0], "SwapTotal") == 0)
+                                swap_total = atoi (tokens[len-2]);
+                        else if (g_strcmp0 (tokens[0], "Active(anon)") == 0)
+                                active = atoi (tokens[len-2]);
+                }
+                g_strfreev (tokens);
+        }
+
+        /* first check if we even have swap, if not consider all swap space used */
+        if (swap_total == 0) {
+                g_debug ("no swap space found");
+                percentage = 100.0f;
+                goto out;
+        }
+        /* work out how close to the line we are */
+        if (swap_free > 0 && active > 0)
+                percentage = (active * 100) / swap_free;
+        g_debug ("total swap available %i kb, active memory %i kb (%.1f%%)", swap_free, active, percentage);
+out:
+        g_free (contents);
+        g_strfreev (lines);
+        return percentage;
+}
+
+/* adapted from upower-0.9 branch */
+static gboolean
+linux_check_enough_swap (void)
+{
+        gfloat waterline = 98.0f; /* 98% */
+        gfloat used_swap = linux_get_used_swap();
+
+        if (used_swap < waterline) {
+                        g_debug ("enough swap to hibernate");
+                        return TRUE;
+                } else {
+                        g_debug ("not enough swap to hibernate");
+                        return FALSE;
+                }
+
+        g_debug ("should not hit this in linux_check_enough_swap");
+        return FALSE;
+}
+
+
+static gboolean
+linux_supports_sleep_state (const gchar *state)
+{
+        gboolean ret = FALSE;
+        gchar *command;
+        GError *error = NULL;
+        gint exit_status;
+
+        /* run script from pm-utils */
+        command = g_strdup_printf ("/usr/bin/pm-is-supported --%s", state);
+        g_debug ("excuting command: %s", command);
+        ret = g_spawn_command_line_sync (command, NULL, NULL, &exit_status, &error);
+        if (!ret) {
+                g_warning ("failed to run script: %s", error->message);
+                g_error_free (error);
+                goto out;
+        }
+        ret = (WIFEXITED(exit_status) && (WEXITSTATUS(exit_status) == EXIT_SUCCESS));
+
+out:
+        g_free (command);
+
+        return ret;
+}
+
+gboolean
+ck_system_can_suspend (void)
+{
+        return linux_supports_sleep_state ("suspend");
+}
+
+gboolean
+ck_system_can_hibernate (void)
+{
+        if (linux_supports_sleep_state ("hibernate"))
+		return linux_check_enough_swap() ;
+        return FALSE;
+}
+
+gboolean
+ck_system_can_hybrid_sleep (void)
+{
+        return linux_supports_sleep_state ("suspend-hybrid");
+}
+
+static gboolean
+ck_selinux_open(void)
+{
+#ifdef HAVE_SELINUX
+        TRACE ();
+
+        if (label_hnd)
+                return TRUE;
+
+        if (is_selinux_enabled() <= 0)
+                return TRUE;
+
+        label_hnd = selabel_open(SELABEL_CTX_FILE, NULL, 0);
+        if (label_hnd) {
+                return TRUE;
+        } else {
+                g_info ("Failed to open selabel handle, reason was: %s", strerror(errno));
+                errno = 0;
+
+                // do not fail in permissive mode
+                return (security_getenforce() == 1) ? FALSE : TRUE;
+        }
+#endif
+
+        return TRUE;
+}
+
+static void
+ck_selinux_close(void)
+{
+#ifdef HAVE_SELINUX
+        if (label_hnd) {
+                selabel_close(label_hnd);
+                label_hnd = NULL;
+        }
+#endif
+}
+
+static gchar*
+ck_selinux_lookup_context(const gchar *dest)
+{
+#ifdef HAVE_SELINUX
+        int rc;
+        GStatBuf st;
+        mode_t mode = 0;
+        security_context_t con;
+        gchar *constr;
+
+        if (!label_hnd)
+                return NULL;
+
+        errno = 0;
+        memset(&st, 0, sizeof(st));
+        rc = g_lstat(dest, &st);
+        if (rc == 0)
+                mode = st.st_mode;
+        else if (errno != ENOENT)
+                return NULL;
+
+        errno = 0;
+        rc = selabel_lookup_raw(label_hnd, &con, dest, mode);
+        if (rc < 0 && errno != ENOENT) {
+                errno = 0;
+                return NULL;
+        }
+
+        constr = g_strdup(con);
+        freecon(con);
+
+        return constr;
+#endif
+
+        return NULL;
+}
+
+gboolean
+ck_make_tmpfs (guint uid, guint gid, const gchar *dest)
+{
+#ifdef HAVE_SYS_MOUNT_H
+        gchar        *opts;
+        gchar        *context;
+        int           result;
+
+        TRACE ();
+
+        context = ck_selinux_lookup_context(dest);
+        if (context) {
+                opts = g_strdup_printf ("mode=0700,uid=%d,rootcontext=%s", uid, context);
+        } else {
+                opts = g_strdup_printf ("mode=0700,uid=%d", uid);
+        }
+
+        g_debug ("mounting tmpfs. uid=%d, gid=%d, dest=%s, opts=%s", uid, gid, dest, opts);
+        result = mount("none", dest, "tmpfs", 0, opts);
+
+        g_free (context);
+        g_free (opts);
+
+        if (result == 0) {
+                return TRUE;
+        } else {
+                g_info ("Failed to create tmpfs mount, reason was: %s", strerror(errno));
+                errno = 0;
+                return FALSE;
+        }
+#endif
+
+        return FALSE;
+}
+
+gboolean
+ck_remove_tmpfs (guint uid, const char* dest)
+{
+#ifdef HAVE_SYS_MOUNT_H
+        int           result;
+
+        TRACE ();
+
+        result = umount2(dest, MNT_DETACH);
+
+        if (result == 0) {
+                return TRUE;
+        }
+
+        g_info ("Failed to unmount tmpfs mount, reason was: %s", strerror(errno));
+        errno = 0;
+#endif
+
+        return FALSE;
+}
+
+#ifdef HAVE_SYS_VT_SIGNAL
+/* For the moment this is Linux only.
+ * Returns the vt file descriptor or < 0 on failure.
+ * /sys/class/tty/tty0/active on Linux
+ */
+gint
+ck_get_vt_signal_fd (void)
+{
+        gint fd;
+        const char *errmsg = NULL;
+
+        errno = 0;
+        /* Open the magic Linux location */
+        fd = open ("/sys/class/tty/tty0/active", O_RDONLY);
+        if (fd < 0) {
+                errmsg = g_strerror (errno);
+                g_error ("ck_get_vt_signal_fd: Error opening sys file: %s",
+                         errmsg);
+        }
+
+        return fd;
+}
+
+static gboolean
+poll_vt_fd (gint sys_fd)
+{
+        struct pollfd fds[1];
+        const char *errmsg = NULL;
+
+        errno = 0;
+        fds[0].fd = sys_fd;
+        fds[0].events = POLLPRI;
+
+        for(;;) {
+                g_debug ("poll_vt_fd: polling");
+                if (poll(fds, 1, -1) < 0) {
+                        errmsg = g_strerror (errno);
+
+                        /* Error handling */
+                        if (errno == EINTR || errno == EAGAIN) {
+                                g_debug ("poll_vt_fd: Interrupted while waiting for vt event: %s",
+                                          errmsg);
+                                /* try again */
+                        } else {
+                                g_error ("poll_vt_fd: Error polling for vt event: %s",
+                                         errmsg);
+                                return FALSE;
+                        }
+                } else if (fds[0].revents & POLLPRI) {
+                        /* There's something we care about! */
+                        return TRUE;
+                } else {
+                        /* Something we don't care about */
+                }
+        }
+
+        return FALSE;
+}
+
+static gint
+read_from_vt_fd (gint sys_fd)
+{
+        gint ret = -1;
+        char new_vt[32];
+        ssize_t bytes_read = 0;
+
+        new_vt[31] = '\0';
+
+        while (bytes_read == 0) {
+                errno = 0;
+
+                bytes_read = read (sys_fd, &new_vt, 30);
+                if (bytes_read == -1 && (errno == EAGAIN || errno == EINTR)) {
+                        g_debug ("read_from_vt_fd: Interrupted while reading from sys_fd: %s",
+                                 g_strerror (errno));
+
+                        bytes_read = 0;
+                        /* try again */
+                } else if (bytes_read == -1) {
+                        g_error ("read_from_vt_fd: Error while reading from sys_fd: %s",
+                                 g_strerror (errno));
+                        ret = -1;
+                } else {
+                        gchar unused[32];
+                        gint  vty_num;
+
+                        g_debug ("read_from_vt_fd: got %s", new_vt);
+
+                        if(sscanf (new_vt, "%31[a-z-A-Z]%d", unused, &vty_num) == 2) {
+                                g_debug ("read_from_vt_fd: parsed as %s %d", unused, vty_num);
+                                ret = vty_num;
+                        }
+
+                        /* back to the beginning of the fd */
+                        if (lseek(sys_fd, 0,SEEK_SET) < 0)
+                        {
+                                g_debug ("read_from_vt_fd: error seeking to beginning of file %s",
+                                         g_strerror (errno));
+                                ret = -1;
+                        }
+                }
+        }
+
+        return ret;
+}
+
+/*
+ * Returns FALSE if something went wrong with reading/polling the
+ * vt fd for VT changes.
+ */
+gboolean
+ck_wait_for_console_switch (gint sys_fd, gint32 *num)
+{
+        gint new_vt = -1;
+
+        g_debug ("ck_wait_for_console_switch: sys_fd opened");
+
+        /* Poll for changes */
+        if (poll_vt_fd (sys_fd)) {
+                g_debug ("ck_wait_for_console_switch: poll_vt_fd returned");
+                /* Read the changes */
+                new_vt = read_from_vt_fd (sys_fd);
+                if (new_vt >= 0) {
+                        g_debug ("ck_wait_for_console_switch: read successful");
+                        /* success, update */
+                        if (num != NULL) {
+                                *num = new_vt;
+                        }
+                }
+        }
+
+        return new_vt < 0 ? FALSE : TRUE;
+}
+#endif /* HAVE_SYS_VT_SIGNAL */
+
+gboolean
+ck_sysdeps_init (void)
+{
+        return ck_selinux_open();
+}
+
+void
+ck_sysdeps_fini (void)
+{
+        ck_selinux_close();
+}
