@@ -26,16 +26,12 @@
 #include <sys/types.h>
 #include <pwd.h>
 #include <string.h>
-
+#include <libintl.h>
 #include <locale.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
-
-#define DBUS_API_SUBJECT_TO_CHANGE
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
+#include <gio/gio.h>
 
 #define CK_NAME      "org.freedesktop.ConsoleKit"
 #define CK_PATH      "/org/freedesktop/ConsoleKit"
@@ -47,95 +43,93 @@
 #define CK_SESSION_INTERFACE "org.freedesktop.ConsoleKit.Session"
 
 static gboolean
-get_uint (DBusGProxy *proxy,
+get_value (GDBusProxy *proxy,
+           const char *method,
+           const char *variant_type,
+           gpointer    val)
+{
+        GError   *error;
+        GVariant *res;
+
+        error = NULL;
+        res = g_dbus_proxy_call_sync (proxy,
+                                      method,
+                                      g_variant_new ("()"),
+                                      G_DBUS_CALL_FLAGS_NONE,
+                                      6000,
+                                      NULL,
+                                      &error);
+        if (res == NULL) {
+                g_debug ("%s failed: %s", method, error->message);
+                g_clear_error (&error);
+                return FALSE;
+        } else {
+                g_variant_get (res, variant_type, val, NULL);
+                g_variant_unref (res);
+        }
+
+        return TRUE;
+}
+
+static gboolean
+get_property (GDBusProxy *proxy,
+              const char *property,
+              const char *variant_type,
+              gpointer    val)
+{
+        GVariant *res;
+
+        res = g_dbus_proxy_get_cached_property (proxy, property);
+
+        if (res == NULL) {
+                g_debug ("failed accessing %s", property);
+                return FALSE;
+        } else {
+                g_variant_get (res, variant_type, val, NULL);
+                g_variant_unref (res);
+        }
+
+        return TRUE;
+}
+
+static gboolean
+get_uint (GDBusProxy *proxy,
           const char *method,
           guint      *val)
 {
-        GError  *error;
-        gboolean res;
-
-        error = NULL;
-        res = dbus_g_proxy_call (proxy,
-                                 method,
-                                 &error,
-                                 G_TYPE_INVALID,
-                                 G_TYPE_UINT, val,
-                                 G_TYPE_INVALID);
-        if (! res) {
-                g_warning ("%s failed: %s", method, error->message);
-                g_error_free (error);
-        }
-
-        return res;
+        return get_value (proxy, method, "(u)", val);
 }
 
 static gboolean
-get_path (DBusGProxy *proxy,
+get_uint_property (GDBusProxy *proxy,
+                   const char *property,
+                   guint      *val)
+{
+        return get_property (proxy, property, "u", val);
+}
+
+static gboolean
+get_path (GDBusProxy *proxy,
           const char *method,
           char      **str)
 {
-        GError  *error;
-        gboolean res;
-
-        error = NULL;
-        res = dbus_g_proxy_call (proxy,
-                                 method,
-                                 &error,
-                                 G_TYPE_INVALID,
-                                 DBUS_TYPE_G_OBJECT_PATH, str,
-                                 G_TYPE_INVALID);
-        if (! res) {
-                g_warning ("%s failed: %s", method, error->message);
-                g_error_free (error);
-        }
-
-        return res;
+        return get_value (proxy, method, "(o)", str);
 }
 
 static gboolean
-get_string (DBusGProxy *proxy,
+get_string (GDBusProxy *proxy,
             const char *method,
             char      **str)
 {
-        GError  *error;
-        gboolean res;
-
-        error = NULL;
-        res = dbus_g_proxy_call (proxy,
-                                 method,
-                                 &error,
-                                 G_TYPE_INVALID,
-                                 G_TYPE_STRING, str,
-                                 G_TYPE_INVALID);
-        if (! res) {
-                g_warning ("%s failed: %s", method, error->message);
-                g_error_free (error);
-        }
-
-        return res;
+        return get_value (proxy, method, "(s)", str);
 }
 
 static gboolean
-get_boolean (DBusGProxy *proxy,
+get_boolean (GDBusProxy *proxy,
              const char *method,
              gboolean   *value)
 {
-        GError  *error;
-        gboolean res;
-
-        error = NULL;
-        res = dbus_g_proxy_call (proxy,
-                                 method,
-                                 &error,
-                                 G_TYPE_INVALID,
-                                 G_TYPE_BOOLEAN, value,
-                                 G_TYPE_INVALID);
-        if (! res) {
-                g_warning ("%s failed: %s", method, error->message);
-                g_error_free (error);
-        }
-
-        return res;
+        return get_value (proxy, method, "(b)", value);
 }
 
 static char *
@@ -167,48 +161,64 @@ get_real_name (uid_t uid)
 }
 
 static void
-list_session (DBusGConnection *connection,
+list_session (GDBusConnection *connection,
               const char      *ssid)
 {
-        DBusGProxy *proxy;
-        guint       uid;
+        GDBusProxy *proxy;
+        guint       uid = 0;
         char       *realname;
         char       *sid;
         char       *lsid;
         char       *session_type;
+        char       *session_class;
+        char       *session_state;
         char       *x11_display;
         char       *x11_display_device;
         char       *display_device;
         char       *remote_host_name;
         char       *creation_time;
         char       *idle_since_hint;
+        char       *runtime_dir;
         gboolean    is_active;
         gboolean    is_local;
         char       *short_sid;
         const char *short_ssid;
+        guint       vtnum;
+        GError     *error = NULL;
 
-        proxy = dbus_g_proxy_new_for_name (connection,
-                                           CK_NAME,
-                                           ssid,
-                                           CK_SESSION_INTERFACE);
+        proxy = g_dbus_proxy_new_sync (connection,
+                                       G_DBUS_PROXY_FLAGS_NONE,
+                                       NULL,
+                                       CK_NAME,
+                                       ssid,
+                                       CK_SESSION_INTERFACE,
+                                       NULL,
+                                       &error);
         if (proxy == NULL) {
+                g_print ("error creating proxy, %s", error->message);
+                g_clear_error (&error);
                 return;
         }
 
         sid = NULL;
         lsid = NULL;
         session_type = NULL;
+        session_class = NULL;
+        session_state = NULL;
         x11_display = NULL;
         x11_display_device = NULL;
         display_device = NULL;
         remote_host_name = NULL;
         creation_time = NULL;
         idle_since_hint = NULL;
+        vtnum = -1;
 
         get_uint (proxy, "GetUnixUser", &uid);
         get_path (proxy, "GetSeatId", &sid);
         get_string (proxy, "GetLoginSessionId", &lsid);
         get_string (proxy, "GetSessionType", &session_type);
+        get_string (proxy, "GetSessionClass", &session_class);
+        get_string (proxy, "GetSessionState", &session_state);
         get_string (proxy, "GetX11Display", &x11_display);
         get_string (proxy, "GetX11DisplayDevice", &x11_display_device);
         get_string (proxy, "GetDisplayDevice", &display_device);
@@ -217,6 +227,8 @@ list_session (DBusGConnection *connection,
         get_boolean (proxy, "IsLocal", &is_local);
         get_string (proxy, "GetCreationTime", &creation_time);
         get_string (proxy, "GetIdleSinceHint", &idle_since_hint);
+        get_string (proxy, "GetXDGRuntimeDir", &runtime_dir);
+        get_uint_property (proxy, "VTNr", &vtnum);
 
         realname = get_real_name (uid);
 
@@ -230,12 +242,29 @@ list_session (DBusGConnection *connection,
                 short_ssid = ssid + strlen (CK_PATH) + 1;
         }
 
-        printf ("%s:\n\tunix-user = '%d'\n\trealname = '%s'\n\tseat = '%s'\n\tsession-type = '%s'\n\tactive = %s\n\tx11-display = '%s'\n\tx11-display-device = '%s'\n\tdisplay-device = '%s'\n\tremote-host-name = '%s'\n\tis-local = %s\n\ton-since = '%s'\n\tlogin-session-id = '%s'",
+        printf ("%s:\n"
+                "\tunix-user = '%d'\n"
+                "\trealname = '%s'\n"
+                "\tseat = '%s'\n"
+                "\tsession-type = '%s'\n"
+                "\tsession-class = '%s'\n"
+                "\tsession-state = '%s'\n"
+                "\tactive = %s\n"
+                "\tx11-display = '%s'\n"
+                "\tx11-display-device = '%s'\n"
+                "\tdisplay-device = '%s'\n"
+                "\tremote-host-name = '%s'\n"
+                "\tis-local = %s\n"
+                "\ton-since = '%s'\n"
+                "\tlogin-session-id = '%s'\n"
+                "\tXDG_RUNTIME_DIR = '%s'",
                 short_ssid,
                 uid,
                 realname,
                 short_sid,
                 session_type,
+                session_class,
+                session_state,
                 is_active ? "TRUE" : "FALSE",
                 x11_display,
                 x11_display_device,
@@ -243,7 +272,11 @@ list_session (DBusGConnection *connection,
                 remote_host_name,
                 is_local ? "TRUE" : "FALSE",
                 creation_time,
-                lsid);
+                lsid,
+                runtime_dir);
+        if (vtnum != 0) {
+                printf ("\n\tVTNr = '%u'", vtnum);
+        }
         if (idle_since_hint != NULL && idle_since_hint[0] != '\0') {
                 printf ("\n\tidle-since-hint = '%s'", idle_since_hint);
         }
@@ -256,105 +289,113 @@ list_session (DBusGConnection *connection,
         g_free (sid);
         g_free (lsid);
         g_free (session_type);
+        g_free (session_class);
+        g_free (session_state);
         g_free (x11_display);
         g_free (x11_display_device);
         g_free (display_device);
+        g_free (runtime_dir);
         g_object_unref (proxy);
 }
 
 static void
-list_sessions (DBusGConnection *connection,
+list_sessions (GDBusConnection *connection,
                const char      *sid)
 {
-        DBusGProxy *proxy;
-        GError     *error;
-        gboolean    res;
-        GPtrArray  *sessions;
-        int         i;
+        GDBusProxy   *proxy;
+        GError       *error;
+        GVariant     *res;
+        GVariantIter *iter;
+
+        proxy = g_dbus_proxy_new_sync (connection,
+                                       G_DBUS_PROXY_FLAGS_NONE,
+                                       NULL,
+                                       CK_NAME,
+                                       sid,
+                                       CK_SEAT_INTERFACE,
+                                       NULL,
+                                       &error);
 
-        proxy = dbus_g_proxy_new_for_name (connection,
-                                           CK_NAME,
-                                           sid,
-                                           CK_SEAT_INTERFACE);
         if (proxy == NULL) {
+                g_print ("error creating proxy, %s", error->message);
+                g_clear_error (&error);
                 return;
         }
 
-        sessions = NULL;
-
         error = NULL;
-        res = dbus_g_proxy_call (proxy,
-                                 "GetSessions",
-                                 &error,
-                                 G_TYPE_INVALID,
-                                 dbus_g_type_get_collection ("GPtrArray", DBUS_TYPE_G_OBJECT_PATH),
-                                 &sessions,
-                                 G_TYPE_INVALID);
-        if (! res) {
+        res = g_dbus_proxy_call_sync (proxy,
+                                      "GetSessions",
+                                      g_variant_new ("()"),
+                                      G_DBUS_CALL_FLAGS_NONE,
+                                      6000,
+                                      NULL,
+                                      &error);
+
+        if (res == NULL) {
                 g_warning ("Failed to get list of sessions for %s: %s", sid, error->message);
-                g_error_free (error);
+                g_clear_error (&error);
                 goto out;
         }
 
-        for (i = 0; i < sessions->len; i++) {
-                char *ssid;
-
-                ssid = g_ptr_array_index (sessions, i);
-
-                list_session (connection, ssid);
-
-                g_free (ssid);
+        g_variant_get (res, "(ao)", &iter);
+        while (g_variant_iter_next (iter, "o", &sid))
+        {
+                list_session (connection, sid);
         }
+        g_variant_iter_free (iter);
+        g_variant_unref (res);
 
-        g_ptr_array_free (sessions, TRUE);
  out:
         g_object_unref (proxy);
 }
 
 static void
-list_seats (DBusGConnection *connection)
+list_seats (GDBusConnection *connection)
 {
-        DBusGProxy *proxy;
-        GError     *error;
-        gboolean    res;
-        GPtrArray  *seats;
-        int         i;
+        GDBusProxy   *proxy;
+        GError       *error;
+        GVariant     *res;
+        GVariantIter *iter;
+        gchar        *path = NULL;
+
+        proxy = g_dbus_proxy_new_sync (connection,
+                                       G_DBUS_PROXY_FLAGS_NONE,
+                                       NULL,
+                                       CK_NAME,
+                                       CK_MANAGER_PATH,
+                                       CK_MANAGER_INTERFACE,
+                                       NULL,
+                                       &error);
 
-        proxy = dbus_g_proxy_new_for_name (connection,
-                                           CK_NAME,
-                                           CK_MANAGER_PATH,
-                                           CK_MANAGER_INTERFACE);
         if (proxy == NULL) {
+                g_print ("error creating proxy, %s", error->message);
+                g_clear_error (&error);
                 return;
         }
 
-        seats = NULL;
-
         error = NULL;
-        res = dbus_g_proxy_call (proxy,
-                                 "GetSeats",
-                                 &error,
-                                 G_TYPE_INVALID,
-                                 dbus_g_type_get_collection ("GPtrArray", DBUS_TYPE_G_OBJECT_PATH),
-                                 &seats,
-                                 G_TYPE_INVALID);
-        if (! res) {
+        res = g_dbus_proxy_call_sync (proxy,
+                                      "GetSeats",
+                                      g_variant_new ("()"),
+                                      G_DBUS_CALL_FLAGS_NONE,
+                                      6000,
+                                      NULL,
+                                      &error);
+
+        if (res == NULL) {
                 g_warning ("Failed to get list of seats: %s", error->message);
-                g_error_free (error);
+                g_clear_error (&error);
                 goto out;
         }
 
-        for (i = 0; i < seats->len; i++) {
-                char *sid;
-
-                sid = g_ptr_array_index (seats, i);
-
-                list_sessions (connection, sid);
-
-                g_free (sid);
+        g_variant_get (res, "(ao)", &iter);
+        while (g_variant_iter_next (iter, "o", &path))
+        {
+                list_sessions (connection, path);
         }
+        g_variant_iter_free (iter);
+        g_variant_unref (res);
 
-        g_ptr_array_free (seats, TRUE);
  out:
         g_object_unref (proxy);
 }
@@ -363,7 +404,7 @@ int
 main (int    argc,
       char **argv)
 {
-        DBusGConnection *connection;
+        GDBusConnection *connection;
 
         GOptionContext *context;
         gboolean        retval;
@@ -374,7 +415,13 @@ main (int    argc,
                 { NULL }
         };
 
-        g_type_init ();
+        /* Setup for i18n */
+        setlocale(LC_ALL, "");
+ 
+#ifdef ENABLE_NLS
+        bindtextdomain(PACKAGE, LOCALEDIR);
+        textdomain(PACKAGE);
+#endif
 
         context = g_option_context_new (NULL);
         g_option_context_add_main_entries (context, entries, NULL);
@@ -394,7 +441,7 @@ main (int    argc,
         }
 
         error = NULL;
-        connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+        connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
         if (connection == NULL) {
                 g_message ("Failed to connect to the D-Bus daemon: %s", error->message);
                 g_error_free (error);
